'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fp = require('lodash/fp');

var _homonyms = require('./homonyms');

var _homonyms2 = _interopRequireDefault(_homonyms);

var _utils = require('../utils');

var _hasher = require('../utils/hasher');

var _combinators = require('../utils/combinators');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var homonymFields = ['_lf_links', '_lf_relations', '_lf_media', '_lf_downloads', '_lf_queries'];

var hashUnitId = (0, _hasher.hashWithField)('_lf_id_fields');
var hashUnitContent = (0, _hasher.hashWithField)('_lf_content_fields');

var dataId = function dataId(u) {
  return u._lf_id_hash || hashUnitId(u);
};
var contentId = function contentId(u) {
  return u._lf_content_hash || hashUnitContent(u);
};

// A single data unit
// Setoid
var equalsOne = (0, _fp.curry)(function (a, b) {
  return (0, _fp.isEqual)(dataId(a), dataId(b));
});

// Monoid
var emptyOne = function emptyOne() {
  var dates = { _lf_pubdates: { fetch: (0, _utils.now)() } };
  var markers = { _lf_markers: [] };
  var homonyms = (0, _fp.reduce)(function (memo, h) {
    return (0, _fp.merge)(memo, _defineProperty({}, h, _homonyms2.default.empty()));
  }, {}, homonymFields);

  return (0, _fp.mergeAll)([dates, markers, homonyms]);
};

var concatOne = (0, _fp.curry)(function (a, b) {
  var homonyms = (0, _fp.reduce)(function (memo, h) {
    return (0, _fp.merge)(memo, _defineProperty({}, h, _homonyms2.default.concat(a[h] || _homonyms2.default.empty(), b[h] || _homonyms2.default.empty())));
  }, {}, homonymFields);
  var markers = {
    _lf_markers: (0, _fp.uniq)((0, _fp.concat)(a._lf_markers || [], b._lf_markers || []))
  };
  return (0, _fp.mergeAll)([a, b, homonyms, markers]);
});

// Data Units
// Setoid
var equals = (0, _fp.curry)(function (xs, ys) {
  var elems = (0, _fp.zip)((0, _fp.sortBy)(dataId, xs), (0, _fp.sortBy)(dataId, ys));
  return (0, _fp.isEqual)((0, _fp.size)(xs), (0, _fp.size)(ys)) && (0, _fp.every)(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        x = _ref2[0],
        y = _ref2[1];

    return equalsOne(x, y);
  }, elems);
});

// Monoid
var empty = (0, _fp.constant)([]);
var concat = (0, _utils.deepConcatWith)(equalsOne, concatOne);

// Functor
var fmap = _fp.map;
var fmapAsync = _combinators.mapP;
var fmapHomonym = (0, _fp.curry)(function (field, f, xs) {
  return fmap(function (u) {
    return concatOne(u, _defineProperty({}, field, _homonyms2.default.fmap(f, u[field])));
  }, xs);
});
var fmapHomonymAsync = (0, _fp.curry)(function (field, f, xs) {
  return fmapAsync(function (u) {
    return _homonyms2.default.fmapAsync(f, u[field]).then(function (ys) {
      return concatOne(u, _defineProperty({}, field, ys));
    });
  }, xs);
});

// Foldable
var foldr = _fp.reduce;
var fold = foldr(concat, empty());
var foldMap = (0, _fp.curry)(function (f, a) {
  return (0, _fp.flow)([fmap(f), fold])(a);
});

// Applicative
var pure = _utils.arrayify;
var apply = function apply(fs, xs) {
  return (0, _fp.map)(function (x) {
    return (0, _fp.reduce)(function (memo, f) {
      return f(memo);
    }, x, fs);
  }, xs);
};

// Combinators
var filter = (0, _fp.curry)(function (p, a) {
  return foldMap(function (x) {
    if (p(x)) {
      return (0, _utils.arrayify)(x);
    }
    return empty();
  }, a);
});

// Hashing
var hashOne = function hashOne(u) {
  var hashes = (0, _fp.reduce)(function (memo, h) {
    return (0, _fp.merge)(memo, _defineProperty({}, h, _homonyms2.default.hash(u[h])));
  }, { _lf_id_hash: dataId(u), _lf_content_hash: contentId(u) }, homonymFields);
  return concatOne(u, hashes);
};
var hash = fmap(hashOne);

exports.default = {
  homonymFields: homonymFields,
  dataId: dataId,
  contentId: contentId,

  equalsOne: equalsOne,
  emptyOne: emptyOne,
  concatOne: concatOne,

  equals: equals,
  empty: empty,
  concat: concat,
  fmap: fmap,
  fmapAsync: fmapAsync,
  fmapHomonym: fmapHomonym,
  fmapHomonymAsync: fmapHomonymAsync,
  foldr: foldr,
  fold: fold,
  foldMap: foldMap,
  pure: pure,
  apply: apply,
  filter: filter,

  hashOne: hashOne,
  hash: hash
};