'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mapRelationsAsync = exports.mapMediaAsync = exports.mapLinksAsync = exports.mapDownloadsAsync = exports.fmapRelationsAsync = exports.fmapMediaAsync = exports.fmapLinksAsync = exports.fmapDownloadsAsync = exports.fmapDataHomonymAsync = exports.queriesByType = exports.filterQueries = exports.filterData = exports.filter = exports.fmapQueriesAsync = exports.fmapDataAsync = exports.fmapQueries = exports.fmapData = exports.fmapAsync = exports.fmap = exports.concatQueries = exports.concatData = exports.concat = exports.empty = exports.equalsQueries = exports.equalsData = exports.equals = undefined;

var _fp = require('lodash/fp');

var _homonyms = require('./homonyms');

var _homonyms2 = _interopRequireDefault(_homonyms);

var _data = require('./data');

var _data2 = _interopRequireDefault(_data);

var _combinators = require('../utils/combinators');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Setoid
var equals = exports.equals = (0, _fp.curry)(function (e1, e2) {
  return _data2.default.equals(e1.data, e2.data) && _homonyms2.default.equals(e1.queries, e2.queries);
});

var equalsData = exports.equalsData = (0, _fp.curry)(function (e1, e2) {
  return _data2.default.equals(e1.data, e2.data);
});
var equalsQueries = exports.equalsQueries = (0, _fp.curry)(function (e1, e2) {
  return _homonyms2.default.equals(e1.queries, e2.queries);
});

// Monoid
var empty = exports.empty = function empty() {
  return { data: _data2.default.empty(), queries: _homonyms2.default.empty() };
};

var concat = exports.concat = (0, _fp.curry)(function (e1, e2) {
  var data = _data2.default.concat(e1.data || _data2.default.empty(), e2.data || _data2.default.empty());
  var queries = _homonyms2.default.concat(e1.queries || _homonyms2.default.empty(), e2.queries || _homonyms2.default.empty());
  return { data: data, queries: queries };
});

var concatData = exports.concatData = (0, _fp.curry)(function (data, e) {
  return concat(e, { data: data });
});
var concatQueries = exports.concatQueries = (0, _fp.curry)(function (queries, e) {
  return concat(e, { queries: queries });
});

// Functor
var fmap = exports.fmap = (0, _fp.curry)(function (f1, f2, e) {
  var data = _data2.default.fmap(f1, e.data || _data2.default.empty());
  var queries = _homonyms2.default.fmap(f2, e.queries || _homonyms2.default.empty());
  return { data: data, queries: queries };
});
var fmapAsync = exports.fmapAsync = (0, _fp.curry)(function (f1, f2, e) {
  return (0, _combinators.mapP)([_data2.default.fmapAsync(f1, e.data || _data2.default.empty()), _homonyms2.default.fmapAsync(f2, e.queries || _homonyms2.default.empty())]).then((0, _fp.zipObject)(['data', 'queries']));
});

var fmapData = exports.fmapData = (0, _fp.curry)(function (f, e) {
  return fmap(f, _fp.identity, e);
});
var fmapQueries = exports.fmapQueries = (0, _fp.curry)(function (f, e) {
  return fmap(_fp.identity, f, e);
});
var fmapDataAsync = exports.fmapDataAsync = (0, _fp.curry)(function (f, e) {
  return _data2.default.fmapAsync(f, e.data || _data2.default.empty()).then(function (data) {
    return { data: data, queries: e.queries };
  });
});
var fmapQueriesAsync = exports.fmapQueriesAsync = (0, _fp.curry)(function (f, e) {
  return _homonyms2.default.fmapAsync(f, e.queries || _homonyms2.default.empty()).then(function (queries) {
    return { data: e.data, queries: queries };
  });
});

// Combinators
var filter = exports.filter = (0, _fp.curry)(function (p1, p2, e) {
  var data = _data2.default.filter(p1, e.data);
  var queries = _homonyms2.default.filter(p2, e.queries);
  return { data: data, queries: queries };
});

var filterData = exports.filterData = (0, _fp.curry)(function (p, e) {
  return filter(p, (0, _fp.constant)(true), e);
});
var filterQueries = exports.filterQueries = (0, _fp.curry)(function (p, e) {
  return filter((0, _fp.constant)(true), p, e);
});

var queriesByType = exports.queriesByType = (0, _fp.curry)(function (type, e) {
  var pred = (0, _fp.flow)([(0, _fp.property)('type'), (0, _fp.isEqual)(type)]);
  return (0, _fp.flow)([filterQueries(pred), (0, _fp.property)('queries'), (0, _fp.map)('term')])(e);
});

var fmapDataHomonymAsync = exports.fmapDataHomonymAsync = (0, _fp.curry)(function (field, f, envelope) {
  return _data2.default.fmapHomonymAsync(field, f, envelope.data).then(function (data) {
    return { data: data, queries: envelope.queries };
  });
});

var fmapDownloadsAsync = exports.fmapDownloadsAsync = fmapDataHomonymAsync('_lf_downloads');
var fmapLinksAsync = exports.fmapLinksAsync = fmapDataHomonymAsync('_lf_links');
var fmapMediaAsync = exports.fmapMediaAsync = fmapDataHomonymAsync('_lf_media');
var fmapRelationsAsync = exports.fmapRelationsAsync = fmapDataHomonymAsync('_lf_relations');
// FIXME: Deprecated!!
var mapDownloadsAsync = exports.mapDownloadsAsync = fmapDownloadsAsync;
var mapLinksAsync = exports.mapLinksAsync = fmapLinksAsync;
var mapMediaAsync = exports.mapMediaAsync = fmapMediaAsync;
var mapRelationsAsync = exports.mapRelationsAsync = fmapRelationsAsync;

exports.default = {
  equals: equals,
  equalsData: equalsData,
  equalsQueries: equalsQueries,
  empty: empty,
  concat: concat,
  concatData: concatData,
  concatQueries: concatQueries,
  fmap: fmap,
  fmapData: fmapData,
  fmapQueries: fmapQueries,
  filter: filter,
  filterData: filterData,
  filterQueries: filterQueries,
  queriesByType: queriesByType,
  fmapDataHomonymAsync: fmapDataHomonymAsync,
  fmapDownloadsAsync: fmapDownloadsAsync,
  fmapLinksAsync: fmapLinksAsync,
  fmapMediaAsync: fmapMediaAsync,
  fmapRelationsAsync: fmapRelationsAsync,

  // Deprecated:
  mapDownloadsAsync: mapDownloadsAsync,
  mapLinksAsync: mapLinksAsync,
  mapMediaAsync: mapMediaAsync,
  mapRelationsAsync: mapRelationsAsync
};