'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fp = require('lodash/fp');

var _utils = require('../utils');

var _hasher = require('../utils/hasher');

var _combinators = require('../utils/combinators');

var hashHomonymId = (0, _hasher.hashKeys)(['type', 'term']);

var homId = function homId(h) {
  return h._lf_id_hash || hashHomonymId(h);
};

// A single Homonym
// Setoid
var equalsOne = (0, _fp.curry)(function (a, b) {
  return (0, _fp.isEqual)(homId(a), homId(b));
});

// Monoid
var emptyOne = (0, _fp.constant)({});
var concatOne = _fp.merge;

// A list of homonyms
// Setoid
var equals = (0, _fp.curry)(function (xs, ys) {
  var elems = (0, _fp.zip)((0, _fp.sortBy)(homId, xs), (0, _fp.sortBy)(homId, ys));
  return (0, _fp.isEqual)((0, _fp.size)(xs), (0, _fp.size)(ys)) && (0, _fp.every)(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        x = _ref2[0],
        y = _ref2[1];

    return equalsOne(x, y);
  }, elems);
});

// Monoid
var empty = (0, _fp.constant)([]);
var concat = (0, _utils.deepConcatWith)(equalsOne, concatOne);

// Functor
var fmap = _fp.map;
var fmapAsync = _combinators.mapP;

// Foldable
var foldr = _fp.reduce;
var fold = foldr(concat, empty());
var foldMap = (0, _fp.curry)(function (f, a) {
  return (0, _fp.flow)([fmap(f), fold])(a);
});

// Combinators
var filter = (0, _fp.curry)(function (p, a) {
  return foldMap(function (x) {
    if (p(x)) {
      return (0, _utils.arrayify)(x);
    }
    return empty();
  }, a);
});

// Hashing
var hashOne = function hashOne(h) {
  return concatOne(h, { _lf_id_hash: homId(h) });
};
var hash = fmap(hashOne);

exports.default = {
  homId: homId,

  equalsOne: equalsOne,
  emptyOne: emptyOne,
  concatOne: concatOne,

  equals: equals,
  empty: empty,
  concat: concat,
  fmap: fmap,
  fmapAsync: fmapAsync,
  foldr: foldr,
  fold: fold,
  foldMap: foldMap,

  filter: filter,
  hashOne: hashOne,
  hash: hash
};