'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.liftManyA2 = exports.liftA2 = exports.apply = exports.pure = exports.fmap = exports.id = exports.options = exports.load = exports.list = exports.listNodeModules = exports.listPackageJson = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fp = require('lodash/fp');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _combinators = require('./utils/combinators');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var mapObj = _fp.map.convert({ cap: false });

/**
 * List all littlefork plugins from the local package.json.
 * @returns {Array.<String>} dependencies A list of plugin names.
 */
var listPackageJson = exports.listPackageJson = (0, _fp.flow)([
// eslint-disable-next-line import/no-dynamic-require, global-require
function () {
  return require('' + _path2.default.join(process.cwd(), 'package'));
}, (0, _fp.property)('dependencies'), _fp.keys, (0, _fp.filter)((0, _fp.startsWith)('littlefork-plugin'))]);

/**
 * List all littlefork plugins found in the `node_modules` directory.
 * @returns {Array.<String>} dependencies A list of plugin names.
 */
var listNodeModules = exports.listNodeModules = (0, _fp.flow)([function () {
  return (0, _fs.readdirSync)('' + _path2.default.join(process.cwd(), 'node_modules'));
}, (0, _fp.filter)((0, _fp.startsWith)('littlefork-plugin'))]);

/**
 * The default listing of plugins. Basically if it's in node_modules, it is
 * available.
 * @returns {Array.<String>} dependencies A list of plugin names.
 */
var list = exports.list = listNodeModules;

/**
 * Load all plugins available for this littlefork installation.
 *
 * @param {Array.<string>} dependencies A list of names of plugins to load.
 * @throws If any dependency in `package.json` isn't installed or a plugin in
 * the config isn't available.
 * @returns {Array<Array.<Object>, Array.<string>>} The plugins object and a
 * list of modules that could not be loaded.
 * @example
 * const [plugins, missing] = load(list());
 */
var load = exports.load = function load(deps) {
  return (0, _fp.reduce)(function (_ref, name) {
    var _ref2 = _slicedToArray(_ref, 2),
        ts = _ref2[0],
        ms = _ref2[1];

    var moduleName = process.cwd() + '/node_modules/' + name;
    var module = {};

    try {
      // eslint-disable-next-line global-require, import/no-dynamic-require
      module = require(moduleName);
    } catch (e) {
      return [ts, (0, _fp.concat)(ms, [name])];
    }

    return [(0, _fp.merge)(ts, module.plugins || {}), ms];
  }, [{}, []], deps);
};

/**
 * Return all available options for all available plugins.
 *
 * @returns {Object} options The options of every plugin.
 */
var options = exports.options = (0, _fp.flow)([list, load, _fp.first, mapObj(function (p) {
  return p.argv;
}), _fp.compact]);

// A littlefork plugin has one of the following types:
//    a -> Future a
//    a -> a
// Plugins that don't return a future, are coerced into a version with a future.
// Each plugin is an instance of a Functor and Applicative Functor.

// future :: a -> Future a
// This just a shorthand function, makes for concise code later on.
var future = _bluebird2.default.resolve;

/**
 * A promised identity function.
 *
 * `id :: a -> Future a`
 *
 * @param {Any} The value to returns.
 * @returns {Promise.<Any>} A promise of the value that was supplied.
 */
var id = exports.id = future;

/**
 * Map a function over a Functor
 *
 * `fmap :: Functor f => (a -> Future b) -> f (Future a) -> Future b`
 * `fmap :: Functor f => (a -> b) -> f a -> Future b`
 *
 * @param {Function} f The function to apply to the Functor.
 * @param {Promise|Any} p The functor value to map.
 * @returns {Promise} A promise resolving to the value of p mapped over f.
 * @example
 * const p = () => Promise.resolve(1);
 * const f = v => v + 1;
 * fmap(f, p);  // Returns a promise resolving to 2.
 */
var fmap = exports.fmap = (0, _fp.curry)(function (f, p) {
  return future(p).then(f);
});

/**
 * Lift a value into an applicative.
 *
 * `pure :: Applicative f => a -> f (Future a)`
 *
 * @param {Any} a The value to lift.
 * @returns {Promise} A promise that resolves to a.
 */
var pure = exports.pure = future;

/**
 * Apply a function wrapped in a promise to a promisified value.
 *
 * `apply :: Applicative f => f (a -> Future b) -> f (Future a) -> f (Future b)`
 * `apply :: Applicative f => f (a -> b) -> f a -> f (Future b)`
 *
 * @param {Promise.<Function>} pf A promise that resolves to a function.
 * @param {Promise.<Any>} p A promise that resolves to a value.
 * @returns {Promise.<Any>} A promise resolving to p applied to the function
 * that pf resolves to.
 * @example
 * const pf = Promise.resolve(v => v + 1);
 * const p = Promise.resolve(1);
 * apply(pf, p); // Returns a promise resolving to 2.
 */
var apply = exports.apply = (0, _fp.curry)(function (pf, p) {
  return fmap(function (f) {
    return fmap(f, p);
  }, pf);
});

/**
 * Lift a binary function over two Applicative.
 *
 * `liftA2 :: Applicative f => f (a -> b -> Future c) -> f (Future a)
 *                             -> f (Future b) -> f (Future c)`
 * `liftA2 :: Applicative f => f (a -> b -> Future c) -> f a -> f b -> f (Future c)`
 *
 * @param {Function.<Any, Any>} f A binary function.
 * @param {Promise.<Any>} a A promise that resolves to a value.
 * @param {Promise.<Any>} b A promise that resolves to a value.
 * @returns {Promise.<Any>} The value that f returns when applied to a and b.
 * @example
 * const a = Promise.resolve(envelope);
 * const b = Promise.resolve(env);
 * liftA2(plugin, a, b); // Calls plugin with the value that a and b resolve to.
 */
var liftA2 = exports.liftA2 = (0, _fp.curry)(function (f, a, b) {
  return apply(fmap(function (x) {
    return function (y) {
      return f(x, y);
    };
  }, a), b);
});

/**
 * Lift many binary functions over two Applicatives.
 *
 * ```
 * liftManyA2 :: Applicative f => [f (a -> b -> Future c)] -> f (Future a)
 *                                -> f (Future b) -> f (Future c)
 * liftManyA2 :: Applicative f => [f (a -> b -> Future c)] -> f a -> f b
 *                                -> f (Future c)
 * ```
 * @param {Array.<Function>} fs A list of binary functions.
 * @param {Promise.<Any>} a A promise that resolves to a value.
 * @param {Promise.<Any>} b A promise that resolves to a value.
 * @returns {Promise.<Any>} The value that that returns when reducing `a` and
 * `b` over `fs`.
 * @example
 * const a = Promise.resolve(envelope);
 * const b = Promise.resolve(env);
 * liftManyA2([f1, f2], a, b); // f1(a,b).then(r => f2(r, b)).then(...)
 */
var liftManyA2 = exports.liftManyA2 = (0, _fp.curry)(function (fs, a, b) {
  return (0, _combinators.reduceP)(function (memo, f) {
    return liftA2(f, memo, b);
  }, a, fs);
});

exports.default = {
  list: list,
  listPackageJson: listPackageJson,
  listNodeModules: listNodeModules,
  load: load,
  options: options,
  id: id,
  fmap: fmap,
  pure: pure,
  apply: apply,
  liftA2: liftA2,
  liftManyA2: liftManyA2
};