'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mapQueriesAsync = exports.retry = exports.promisify = exports.reduceP = exports.mapP = exports.flowP = exports.composeP = undefined;

var _fp = require('lodash/fp');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _envelope = require('../data/envelope');

var _data = require('../data/data');

var _data2 = _interopRequireDefault(_data);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// composeP :: (b -> Future c) -> (a -> Future b) -> Future a -> Future c
// composeP :: (b -> Future c) -> (a -> Future b) -> a -> Future c
// This function is overloaded to allow async and non async input.
var composeP = exports.composeP = (0, _fp.curry)(function (f, g, a) {
  return _bluebird2.default.resolve(a).then(g).then(f);
});

var flowP = exports.flowP = (0, _fp.curry)(function (fs, x) {
  return (0, _fp.reduce)(function (memo, f) {
    return memo.then(f);
  }, _bluebird2.default.resolve(x), fs);
});

var mapP = exports.mapP = (0, _fp.curry)(function (f, xs) {
  return _bluebird2.default.map(xs, f, { concurrency: 1 });
});

/**
 * Reduce a list of values over an applicative.
 *
 * ```
 * reduceP :: Applicative f => (Future b -> a -> Future b) -> f (Future b)
 *                             -> [a] -> f (Future b)
 * reduceP :: Applicative f => (b -> a -> f b) -> b -> [a] -> f b
 * ```
 * @param {Function.<Promise, any>} f The reduce function, takes a promise as
 * accumulator and a value.
 * @param {Promise} acc The initial accumulator value.
 * @param {Array.<any>} xs A list of values to reduce.
 * @example
 * reduceP((memo, a) => memo.then(r => r + a), 0, [1,2,3]);
 */
var reduceP = exports.reduceP = (0, _fp.curry)(function (f, acc, xs) {
  return _bluebird2.default.reduce(xs, f, acc);
});

var promisify = exports.promisify = function promisify(f) {
  return (0, _fp.flow)([f, _bluebird2.default.resolve]);
};

var retry = exports.retry = function retry(action) {
  var cfg = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var opts = (0, _fp.merge)({ times: 5, delay: 250, delayModifier: 1.5 }, cfg);

  var resolver = function resolver(times, delay) {
    var result = _bluebird2.default.try(action);
    return times <= 0 ? result : result.catch(function () {
      return _bluebird2.default.delay(delay);
    }).then(function () {
      return resolver(times - 1, delay * opts.delayModifier);
    });
  };

  return resolver(opts.times, opts.delay);
};

var mapQueriesAsync = exports.mapQueriesAsync = (0, _fp.curry)(function (f, source, envelope) {
  var e = (0, _envelope.filterQueries)(function (_ref) {
    var type = _ref.type;
    return type === source;
  }, envelope);
  return mapP(function (q) {
    return f(q.term).then(_data2.default.fmap(_data2.default.concatOne({ _lf_queries: [q] })));
  }, e.queries).then(function (ds) {
    return (0, _envelope.concatData)(_data2.default.fold(ds), envelope);
  });
});

exports.default = {
  composeP: composeP,
  flowP: flowP,
  mapP: mapP,
  reduceP: reduceP,
  promisify: promisify,
  retry: retry,
  mapQueriesAsync: mapQueriesAsync
};