'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.md5sum = exports.sha256sum = exports.mkdirP = undefined;

var _fp = require('lodash/fp');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _path = require('path');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_bluebird2.default.promisifyAll(_fs2.default);

var mkdirP = exports.mkdirP = function mkdirP(dir) {
  return _fs2.default.mkdirAsync(dir).catch(function (err) {
    switch (err.code) {
      case 'EEXIST':
        return _bluebird2.default.resolve();
      case 'ENOENT':
        return mkdirP((0, _path.dirname)(dir)).then(function () {
          return mkdirP(dir);
        });
      default:
        throw err;
    }
  });
};

var hashFile = (0, _fp.curry)(function (algorithm, target) {
  var fd = _fs2.default.createReadStream(target);
  var hash = _crypto2.default.createHash(algorithm);
  hash.setEncoding('hex');

  return new _bluebird2.default(function (resolve, reject) {
    fd.on('end', function () {
      hash.end();
      resolve(hash.read());
    });
    fd.on('error', reject);

    fd.pipe(hash);
  });
});

var sha256sum = exports.sha256sum = hashFile('sha256');
var md5sum = exports.md5sum = hashFile('md5');

exports.default = {
  mkdirP: mkdirP,
  sha256sum: sha256sum,
  md5sum: md5sum
};