'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deepConcatWith = exports.arrayify = exports.stringify = exports.printf = exports.tap = exports.now = undefined;

var _fp = require('lodash/fp');

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _util = require('util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var now = exports.now = function now() {
  return _moment2.default.utc().toDate();
};

var tap = exports.tap = (0, _fp.curry)(function (f, x) {
  f((0, _fp.cloneDeep)(x));
  return x;
});

var printf = exports.printf = function printf(x) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var options = (0, _fp.merge)({ colors: true }, opts);
  var f = (0, _fp.flow)([(0, _fp.partialRight)(_util.inspect, [options]), console.log]);
  return tap(f, x);
};

var stringify = exports.stringify = function stringify(s) {
  if ((0, _fp.isString)(s)) {
    return s;
  }
  return JSON.stringify(s);
};

var arrayify = exports.arrayify = function arrayify(a) {
  if ((0, _fp.isArray)(a)) {
    return a;
  }
  return [a];
};

var deepConcatWith = exports.deepConcatWith = (0, _fp.curry)(function (equals, merger, xs, ys) {
  return (0, _fp.reduce)(function (memo, u) {
    var i = (0, _fp.findIndex)(equals(u), memo);
    if (i === -1) return (0, _fp.concat)(memo, u);
    memo[i] = merger(memo[i], u); // eslint-disable-line no-param-reassign
    return memo;
  }, [], (0, _fp.concat)(xs, ys));
});

exports.default = {
  now: now,
  tap: tap,
  printf: printf,
  stringify: stringify,
  arrayify: arrayify,
  deepConcatWith: deepConcatWith
};