# The Littlefork API

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->


- [Plugin Runner](#plugin-runner)
  - [runner](#runner)
- [Plugins](#plugins)
  - [listPackageJson](#listpackagejson)
  - [listNodeModules](#listnodemodules)
  - [list](#list)
  - [load](#load)
  - [options](#options)
  - [id](#id)
  - [fmap](#fmap)
  - [pure](#pure)
  - [apply](#apply)
  - [liftA2](#lifta2)
  - [liftManyA2](#liftmanya2)
- [Units](#units)
- [Envelope](#envelope)
- [Utils](#utils)
  - [Combinators](#combinators)
    - [reduceP](#reducep)
  - [Hasher](#hasher)
  - [Assertions](#assertions)
  - [FS](#fs)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Plugin Runner

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

### runner

Create a runable littlefork object.

Upon calling this function, you receive a runnable littlefork pipeline and
an observable stream. The littlefork pipeline is a function that can be
called without any arguments. It will return a promise that resolves to the
result of the pipeline run. The stream object is used to receive messages
during the pipeline run. It's currently mainly used for logging purposes,
but can be used for more as well.

The stream sends messages with the following types:

-   `log_info`
-   `log_debug`
-   `log_error`
-   `plugin_start`
-   `plugin_end`

**Parameters**

-   `config` **[Object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)** Configuration for a littlefork run.
-   `queryIds` **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[String](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)>** A list of ids to query.

**Examples**

```javascript
const [run, stream] = runner(config, queryIds);

stream.onValue(msg => {
  switch (msg.type) {
    case 'log_info': console.log(msg.msg); break;
    // ... other cases ...
    default: break;
  }
});

run();
```

Returns **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Function](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/function), Observable>** Return a tuple with a function that
runs the pipeline and an observable object, that receives messages during the
pipeline run. The observable object is BaconJS stream and has the full BaconJS
API available.

## Plugins

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

### listPackageJson

List all littlefork plugins from the local package.json.

Returns **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[String](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)>** dependencies A list of plugin names.

### listNodeModules

List all littlefork plugins found in the `node_modules` directory.

Returns **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[String](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)>** dependencies A list of plugin names.

### list

The default listing of plugins. Basically if it's in node_modules, it is
available.

Returns **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[String](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)>** dependencies A list of plugin names.

### load

Load all plugins available for this littlefork installation.

**Parameters**

-   `dependencies` **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)>** A list of names of plugins to load.
-   `deps`  

**Examples**

```javascript
const [plugins, missing] = load(list());
```

-   Throws **any** If any dependency in `package.json` isn't installed or a plugin in
    the config isn't available.

Returns **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)>, [Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String)>>** The plugins object and a
list of modules that could not be loaded.

### options

Return all available options for all available plugins.

Returns **[Object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object)** options The options of every plugin.

### id

A promised identity function.

`id :: a -> Future a`

**Parameters**

-   `The` **Any** value to returns.

Returns **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** A promise of the value that was supplied.

### fmap

Map a function over a Functor

`fmap :: Functor f => (a -> Future b) -> f (Future a) -> Future b`
`fmap :: Functor f => (a -> b) -> f a -> Future b`

**Parameters**

-   `f` **[Function](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/function)** The function to apply to the Functor.
-   `p` **([Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise) | Any)** The functor value to map.

**Examples**

```javascript
const p = () => Promise.resolve(1);
const f = v => v + 1;
fmap(f, p);  // Returns a promise resolving to 2.
```

Returns **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)** A promise resolving to the value of p mapped over f.

### pure

Lift a value into an applicative.

`pure :: Applicative f => a -> f (Future a)`

**Parameters**

-   `a` **Any** The value to lift.

Returns **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)** A promise that resolves to a.

### apply

Apply a function wrapped in a promise to a promisified value.

`apply :: Applicative f => f (a -> Future b) -> f (Future a) -> f (Future b)`
`apply :: Applicative f => f (a -> b) -> f a -> f (Future b)`

**Parameters**

-   `pf` **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;[Function](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/function)>** A promise that resolves to a function.
-   `p` **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** A promise that resolves to a value.

**Examples**

```javascript
const pf = Promise.resolve(v => v + 1);
const p = Promise.resolve(1);
apply(pf, p); // Returns a promise resolving to 2.
```

Returns **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** A promise resolving to p applied to the function
that pf resolves to.

### liftA2

Lift a binary function over two Applicative.

`liftA2 :: Applicative f => f (a -> b -> Future c) -> f (Future a)
                            -> f (Future b) -> f (Future c)`
`liftA2 :: Applicative f => f (a -> b -> Future c) -> f a -> f b -> f (Future c)`

**Parameters**

-   `f` **[Function](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/function)&lt;Any, Any>** A binary function.
-   `a` **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** A promise that resolves to a value.
-   `b` **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** A promise that resolves to a value.

**Examples**

```javascript
const a = Promise.resolve(envelope);
const b = Promise.resolve(env);
liftA2(plugin, a, b); // Calls plugin with the value that a and b resolve to.
```

Returns **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** The value that f returns when applied to a and b.

### liftManyA2

Lift many binary functions over two Applicatives.

    liftManyA2 :: Applicative f => [f (a -> b -> Future c)] -> f (Future a)
                                   -> f (Future b) -> f (Future c)
    liftManyA2 :: Applicative f => [f (a -> b -> Future c)] -> f a -> f b
                                   -> f (Future c)

**Parameters**

-   `fs` **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Function](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/function)>** A list of binary functions.
-   `a` **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** A promise that resolves to a value.
-   `b` **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** A promise that resolves to a value.

**Examples**

```javascript
const a = Promise.resolve(envelope);
const b = Promise.resolve(env);
liftManyA2([f1, f2], a, b); // f1(a,b).then(r => f2(r, b)).then(...)
```

Returns **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;Any>** The value that that returns when reducing `a` and
`b` over `fs`.

## Units

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

## Envelope

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

## Utils

### Combinators

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### reduceP

Reduce a list of values over an applicative.

    reduceP :: Applicative f => (Future b -> a -> Future b) -> f (Future b)
                                -> [a] -> f (Future b)
    reduceP :: Applicative f => (b -> a -> f b) -> b -> [a] -> f b

**Parameters**

-   `f` **[Function](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/function)&lt;[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise), any>** The reduce function, takes a promise as
    accumulator and a value.
-   `acc` **[Promise](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise)** The initial accumulator value.
-   `xs` **[Array](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;any>** A list of values to reduce.

**Examples**

```javascript
reduceP((memo, a) => memo.then(r => r + a), 0, [1,2,3]);
```

### Hasher

### Assertions

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

### FS

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->
