# The Littlefork guide for developers

Littlefork is written in JavaScript and runs on NodeJS. Packages can be
obtained from [NPM](https://npmjs.org). Littlefork is split in several
packages:

- [`littlefork`](https://npmjs.org/package/littlefork)

  The core of Littlefork. It provides the machinery to run plugins and an API
  to make the life of plugin developers easier.

- [`littlefork-cli`](https://npmjs.org/package/littlefork-cli)

  A command line interface for Littlefork. It wraps the core and provides the
  `littlefork` executable.

- [`littlefork-api`](https://npmjs.org/package/littlefork-api)

  A RESTful HTTP API, that wraps the core. It allows to control Littlefork
  using HTTP requests.

- [`littlefork-plugin-*`](https://www.npmjs.org/browse/keyword/littlefork-plugin)

  All the functionality that Littlefork provides is encapsulated in plugins.

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->


- [Plugins](#plugins)
- [The Envelope](#the-envelope)
  - [Data](#data)
  - [Queries](#queries)
- [TODO: Developing plugins](#todo-developing-plugins)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Plugins

A plugin is a function that takes an envelope as input, and returns an
envelope as output. The exact signature of a Littlefork plugin is the
following:

`plugin(Envelope, {log, cfg}) -> Envelope`

The envelope contains all data and queries and every plugin has to return data
with the same interface. We will look at the envelope in more detail
later.

The second argument is a bag of different objects. Currently it provides the
whole configuration object of the run and a logger object. In the future this
can be extended.

Plugins can return either an envelope or a promise that resolves to an
envelope. The Littlefork core accepts both return types. Internally if the
return type is a value, it will be turned into a resolved promise.

Plugins implement an interface for functors and applicatives and offers a few
combinators on top of those abstractions. See the [API guide](./api#plugins)
for details.

## The Envelope

Data and queries are wrapped in an envelope, which is just an object. This is
a minimal valid envelope.

```
{
  data: [],
  queries: []
}
```

Both `data` and `queries` are a list of objects.

The envelope as well as `data` and `queries` implement various interfaces such
as monoids and functors. See the API guide [here](./api#units)
and [here](./api#envelope) for details.

### Data

Plugins create units of data and they are free to extend the format of a data
unit with whatever field they want. Certain fields gold meaning across the
whole of Littlefork, and plugins should not change their meaning. Some fields
are understood only by a few plugins.

- `_lf_id_hash` - A unique identifier for this data unit. It is usually a
  SHA256 hash. If the `_lf_id_fields` field is set, the Littlefork core
  automatically calculates the `_lf_id_hash` for every unit. This is the
  recommended way. Otherwise the plugin has to provide the hash by itself.

- `_lf_id_fields` - A list of field names, that specify the significant fields
  to form an identifier for this unit. Littlefork will take the values for
  those fields to calculate a unique identifier.

  `_lf_id_fields: ['tweet_id']`

- `_lf_content_hash` - A SHA256 hash of the content of the data
  unit. Littlefork uses this field to see if a particular piece of data
  changed. If the `_lf_content_fields` field is et, the Littlefork core
  automatically calculates the `_lf_content_hash` for every unit. This is the
  recommended way. Otherwise the plugin has to provide the hash by
  itself.

- `_lf_content_fields` - A list of field names, that specify the significant
  fields to form the content for this unit. Littlefork will take the values
  for those fields to calculate content integrity.

  `_lf_content_fields: ['tweet']`

- `_lf_pubdates` - An object, that contains various dates around this
  unit. Possible date types are `fetch`, the date and time when the unit was
  fetched, and `source`, the date and time that the unit was created. In the
  future more types of dates can be added. The dates should be JavaScript
  `Date` objects.

  ```
  _lf_pubdates: {
    fetch: Date("2017-03-21T23:22:35.426Z"),
    source: Date("2016-08-14T12:00:35Z")
  }
  ```

- `_lf_links`
- `_lf_media`
- `_lf_relations`
- `_lf_downloads`
_ `_lf_markers`

### Queries

Queries are, the same as data, a list of objects. Each object must have two
fields:

- `type` - What kind of query it is. Plugins will pick all queries they wanna
  operate on based on the type.

- `term` - The actual query. The format of the term depends on the
  type. Queries of type `ddg_search` are simple strings, whereas queries of
  type `mongodb_query` are a MongoDB query description.

Plugins document which query types they expect.

## TODO: Developing plugins
